import { alertService } from '@/helpers/AlertService';
import { FormValues } from '@/types/InterfaceChangePasswordController';
import {
  Form, LoadingService, Modal, ModalService,
} from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';
import { LoginController } from './LoginController';

export class UpdatePasswordController {
  private resetPasswordModal!: Modal;

  public updatePasswordModal!: Modal;

  public loginController = new LoginController();

  public async createUpdatePasswordModal(): Promise<void> {
    this.updatePasswordModal = await ModalService.createFromJson('update-password-modal', true);
  }

  public async openUpdatePasswordModal(): Promise<void> {
    if (this.updatePasswordModal) this.updatePasswordModal.destroy();
    await this.createUpdatePasswordModal();
    this.updatePasswordModal.show();
  }

  public async closeUpdatePasswordModal() {
    if (!this.updatePasswordModal) await this.createUpdatePasswordModal();
    this.updatePasswordModal.destroy();
  }

  public form(): Form {
    const form = Metadata.getInstance<Form>('update-password-modal-form');
    return form;
  }

  public validateForm(): boolean {
    const form = this.form();
    const valid = form.validate();
    return valid;
  }

  public valuesForm(): FormValues {
    const { value } = this.form();
    const email = localStorage.getItem('EMAIL');
    const operatorName = localStorage.getItem('NAME')?.split(' ')[0] || '';
    const operatorId = localStorage.getItem('ID');

    return {
      currentPassword: value.currentPassword,
      newPassword: value.newPassword,
      confirmationPassword: value.confirmationPassword,
      email,
      operatorName,
      operatorId,
    };
  }

  public async executeFormSubmit(): Promise<void> {
    if (!this.validateForm()) return;
    const form = this.valuesForm();
    if (form.newPassword !== form.confirmationPassword) {
      const message = `A senha digitada não corresponde à senha de confirmação. 
      Por favor, certifique-se de que digitou corretamente e tente novamente.`;
      alertService('error', message, 'red');
      return;
    }
    await this.formSubmit(form);
  }

  public async formSubmit({
    currentPassword, newPassword, email, operatorName, operatorId,
  }: FormValues) {
    try {
      LoadingService.show();
      const { data: { error, message } } = await Http.post('/operator/updatePassword',
        {
          currentPassword, newPassword, email, operatorName, operatorId,
        });

      if (error) throw new Error(message);

      alertService('success', message, 'green');

      setTimeout(() => { this.loginController.logout(); }, 2000);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    } finally {
      this.closeUpdatePasswordModal();
      LoadingService.hide();
    }
  }

  public async resetPassword() {
    try {
      const form = this.getInstanceFromResetPasswordForm();
      if (!form) return;
      LoadingService.show();
      const urlParams = new URLSearchParams(window.location.search);
      const token = urlParams.get('token');
      const resetPasswordParams = [form, token];
      const { data } = await Http.post('/operator/resetPassword', resetPasswordParams);

      if (data.error) throw new Error(data.message);
      alertService('alert', 'Senha alterada com sucesso.', 'success');
      setTimeout(() => {
        window.location.href = '/login';
      }, 3000);
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
        if (error.message === 'Token invalido') {
          setTimeout(() => {
            window.location.href = '/login';
          }, 3000);
        }
      }
    } finally {
      LoadingService.hide();
    }
  }

  public getInstanceFromResetPasswordForm() {
    const form = Metadata.getInstance<Form>('reset-password-form');
    if (!form.validate()) return null;
    const password = form.value['reset-password-input'];
    const passwordConfirm = form.value['reset-password-input-confirm'];

    return {
      password,
      passwordConfirm,
    };
  }

  public async openModalResetPassword() {
    if (this.resetPasswordModal) this.closeModalResetPassword();
    this.resetPasswordModal = await ModalService.createFromJson('reset-password-modal', true);
    this.resetPasswordModal.show();
  }

  public closeModalResetPassword(): void {
    this.resetPasswordModal.destroy();
  }

  public async sendResetPasswordEmail() {
    try {
      LoadingService.show();
      const formModalResetPassword: any = this.getChildElement(this.resetPasswordModal, 'reset-password-form');
      const instanceForm = Metadata.getInstance<Form>(formModalResetPassword.name);

      if (!instanceForm.validate()) return;

      const email = instanceForm.value;
      const { data } = await Http.post('/operator/sendResetPasswordEmail', { email });
      if (data.error) throw new Error(data.message);
      this.resetPasswordModal?.destroy();
      alertService('alert', `${data.message}`, 'success');
    } catch (error) {
      if (error instanceof Error) {
        alertService('error', error.message, 'red');
      }
    } finally {
      LoadingService.hide();
    }
  }

  private getChildElement(component: Modal | Form, name: string) {
    const element = component?.children.find((child: any) => child.name === name);
    return element;
  }
}
